# раскомментировать, когда перейдем на разные пароли
import hashlib
import json
import threading
import time
from datetime import datetime

import config
# test
import jsonAndRequest as jsreq
from BotClass import JabberBot

global tasklist

botname = "open_service_request_assigned"

operating_status = '212'
JID = botname + "@" + config.JSERVER
PORT = config.PORT

PASSWORD = hashlib.md5((botname + config.PASSWORD).encode('utf-8')).hexdigest()

print(JID, PASSWORD)

drivers = ["799999999998@ej.sharix-app.org"]
# таким образом хранится список jid, от которых можно получать сообщения этому боту
listen_to = ["open_service_request_booked@ej.sharix-app.org", "open_service_request_pending@ej.sharix-app.org"]

# тут хранится список jid, кому бот может отправлять сообщения в результате обработки заявки
proceed_to = [botname, "open_service_request_accepted@ej.sharix-app.org",
              "open_service_request_declined@ej.sharix-app.org", "open_service_request_forcemajeure@ej.sharix-app.org"]

# максимальное значение попыток обработать заказ
idle_value = 10


def log_handler():
    while True:
        with open(f'{botname}.txt', 'a') as log_file:
            log_file.write(f"{time.strftime('%Y-%m-%d %H:%M:%S')} - {botname} is still running...\n")
        time.sleep(3600)


# обработчик входящих сообщений
def message_handler(conn, mess):
    text = mess.getBody()  # текст сообщения боту
    user = mess.getFrom()  # отправитель сообщения
    if (str(user).split("/")[0]) in listen_to:
        print('order_handler')

        if text is not None:
            orderObj = jsreq.jsonToOrder(text)
            tasklist.append(orderObj)
            bot.bot_log(str(datetime.now()) + " Поступивший заказ успешно добавлен в очередь обработки\n")


def open_service_request_assigned_wait(period, localtask):
    bot.bot_log(botname + " " + str(datetime.now()) + " " + "Заказ ожидает\n")
    time.sleep(period)

    # ВАЖНО! Надо скорее всего через API редактировать заказ, иначе возможна потеря данных
    localtask.title = localtask.title + "1"

    # обязательно данный обработчик должен заканчиваться передачей заказа куда-то на обработку дальше - обратно или другому, иначе оно потеряется
    bot.proceed_status(proceed_to[0], localtask)
    bot.bot_log(botname + " " + str(datetime.now()) + " " + "Заказ подождал и отправлен в очередь\n")


def wait_for_answer(localtask):
    order = localtask
    driver = drivers[0]
    send_message(order, driver)
    # Здесь можно реализовать логику выбора водителя
    while True:
        start_time = time.time()
        if time.time() + 400 - start_time > 300:  ##Логика временная
            if len(answers) > 0:
                answer = answers.pop(0)
                if answer.answer == 'ACCEPT':
                    localtask.status = '221'
                    jsreq.change_to_task(localtask, token)
                    bot.proceed_status(proceed_to[1], localtask)
                    bot.bot_log(botname + " " + str(datetime.now()) + f" Заказ отправлен на {proceed_to[1]}\n")
                elif answer.answer == 'DECLINE':
                    localtask.status = '229'
                    localtask.provider = None
                    jsreq.change_to_task(localtask, token)
                    bot.proceed_status(proceed_to[2], localtask)
                    bot.bot_log(botname + " " + str(datetime.now()) + f" Заказ отправлен на {proceed_to[2]}\n")
                else:
                    bot.bot_log(botname + " " + str(datetime.now()) + f"Ошибка ответа\n")
                break
            else:
                print("Ожидание ответа...")
                time.sleep(3)
        else:
            localtask.status = '229'
            localtask.provider = None
            jsreq.change_to_task(localtask, token)
            bot.proceed_status(proceed_to[2], localtask)
            bot.bot_log(botname + " " + str(datetime.now()) + f" Заказ отправлен на {proceed_to[2]}\n")
            break


def open_service_request_assigned_process(localtask):
    bot.bot_log(botname + " " + str(datetime.now()) + " " + "Заказ в обработчике\n")
    order = localtask
    if order.provider != None:
        wait_thread = threading.Thread(target=wait_for_answer, args=(localtask,))
        wait_thread.start()


def message_answer(conn, mess):
    text = mess.getBody()  # текст сообщения боту
    user = mess.getFrom()  # отправитель сообщения
    if (str(user).split("/")[0]) in drivers:
        print('answer_handler')
        if text is not None:
            answerObj = jsreq.to_answer(text)
            answers.append(answerObj)
            bot.bot_log(str(datetime.now()) + " Поступивший ответ успешно добавлен в очередь обработки\n")


def send_message(order, driver):
    order = json.dumps(order.__dict__)
    bot.send_notification(driver, order)


def open_service_request_assigned():
    log_thread = threading.Thread(target=log_handler)
    log_thread.daemon = True
    log_thread.start()
    while (len(tasklist)):
        localtask = tasklist.pop(0)
        if (localtask.status != operating_status):
            localtask.status = '239'
            jsreq.change_to_task(localtask, token)
            bot.proceed_status(proceed_to[-1], localtask)
            bot.bot_log(botname + " " + str(datetime.now()) + " " + "Заказ не по адресу, перенаправляем на форсмажор\n")
            continue
        else:
            bot.bot_log(botname + " " + datetime.now().strftime('%Y-%m-%d') + " " + "Заказ в очереди на обработке\n")
            t1 = threading.Thread(target=open_service_request_assigned_process, args=([localtask]))
            t1.start()
        # if (localtask.title!="10"):
        #     t1=threading.Thread(target=open_service_request_booked_wait, args=(5,localtask))
        #     t1.start()

        # if (localtask.title=="101"):
        #     t1=threading.Thread(target=open_service_request_booked_process, args=(10,localtask))
        #     t1.start()

        # t1=threading.Thread(target=open_service_request_assigned_process, args=(localtask))
        # t1.start()

        # если никакие обработчики не подошли - отправляем обратно в очередь
        # bot.proceed_status(proceed_to[0],localtask)
        # print("Заказ возвращен в очередь\n")
        # print(tasklist)


# Авторизация и запуск бота

answers = []
# пустой список заказов
tasklist = []

# надо инициализировать tasklist при запуске из API
token = jsreq.requestGetToken(botname, PASSWORD, config.API_URL + "my/api/v1/auth/token/login/")
# ВАЖНО! Надо еще фильтровать список по статусу обработки заявки
tasklist_dbsynce = jsreq.requestGetList(token, config.API_URL + "/dbsynce/api/orders/")

bot = JabberBot(JID, PASSWORD, PORT)
bot.register_handler('message', message_handler)
bot.register_handler('message', message_answer)
bot.start(open_service_request_assigned)

# Добавить функцию поток для ожидания получения заказа.
